// Copyright (c) 2024 The Bitcoin developers
// Distributed under the MIT software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.

import { MockChronikClient } from '../../../../modules/mock-chronik-client';
import { CashtabSettings } from 'config/cashtabSettings';
import CashtabCache from 'config/CashtabCache';
import { cashtabCacheToJSON } from 'helpers';

/**
 * Get expected mock values for chronik client for a given mock wallet
 * Used to support integration testing in Cashtab
 * Default methods may be overwritten in individual unit tests to test special conditions
 * @param {array | object | boolean} wallets Array of wallets stored in localforage.
 * If object, convert to array of length 1.
 * False if user has not yet created a wallet.
 * @param {object} localforage the localforage instance used in your test
 * @param {boolean} apiError Default false. If true, return a mockedChronik that throws errors.
 * @returns {object} mockChronikClient, a mock chronik client instance prepopulated for expected Cashtab API calls
 */
export const initializeCashtabStateForTests = async (
    wallets,
    localforage,
    apiError = false,
) => {
    // Mock successful utxos calls in chronik
    const chronikClient = new MockChronikClient();

    if (wallets === false) {
        // No info to give to chronik, do not populate mocks
        return chronikClient;
        // We do not expect anything in localforage for this case
    }

    wallets = Array.isArray(wallets) ? wallets : [wallets];

    // Set wallets in localforage
    await localforage.setItem('wallets', wallets);

    // All other localforage items will be unset unless the user has customized them
    // Cashtab will use defaults
    // localforage may be modified in individual test cases to test cases of user with
    // non-default settings, cashtabCache, or contactList

    // Mock returns for chronik calls expected in useWallet's update routine for all wallets
    for (const wallet of wallets) {
        prepareMockedChronikCallsForWallet(chronikClient, wallet, apiError);
    }

    return chronikClient;
};

/**
 * Get expected mock values for chronik client for a given mock wallet for a pre 2.0.0 Cashtab user
 * Used to support integration testing in Cashtab
 * Default methods may be overwritten in individual unit tests to test special conditions
 * @param {object | boolean} wallet A mock Cashtab wallet
 * @param {object} localforage the localforage instance used in your test
 * @param {boolean} apiError Default false. If true, return a mockedChronik that throws errors.
 * @returns {object} mockChronikClient, a mock chronik client instance prepopulated for expected Cashtab API calls
 */
export const initializeCashtabStateAtLegacyWalletKeysForTests = async (
    wallet,
    localforage,
    apiError = false,
) => {
    // Mock successful utxos calls in chronik
    const chronikClient = new MockChronikClient();

    if (wallet === false) {
        // No info to give to chronik, do not populate mocks
        return chronikClient;
        // We do not expect anything in localforage for this case
    }

    // Set localforage items. All defaults may be overwritten in a test for
    // specific purposes of the test.
    await localforage.setItem(
        'cashtabCache',
        cashtabCacheToJSON(new CashtabCache()),
    );
    await localforage.setItem('settings', new CashtabSettings());
    // 'contactList' key will be empty if user has never added contacts
    await localforage.setItem('savedWallets', [wallet]);
    await localforage.setItem('wallet', wallet);

    // Mock returns for chronik calls expected in useWallet's update routine
    prepareMockedChronikCallsForWallet(chronikClient, wallet, apiError);

    return chronikClient;
};

/**
 *
 * @param {object} chronikClient a mockedChronikClient object
 * @param {object} wallet a valid cashtab wallet object
 * @param {boolean} apiError true if we want to set api errors in mocked chronik
 * @returns modifies chronikClient in place to have expected API calls for wallet loading available
 */
export const prepareMockedChronikCallsForLegacyWallet = (
    chronikClient,
    wallet,
    apiError = false,
) => {
    // mock chronik endpoint returns
    const CASHTAB_TESTS_TIPHEIGHT = 800000;
    chronikClient.setMock('blockchainInfo', {
        output: apiError
            ? new Error('Error fetching blockchainInfo')
            : { tipHeight: CASHTAB_TESTS_TIPHEIGHT },
    });
    // Mock scriptutxos to match context
    // Cashtab only supports p2pkh addresses
    const CASHTAB_ADDRESS_TYPE = 'p2pkh';
    chronikClient.setScript(CASHTAB_ADDRESS_TYPE, wallet.Path1899.hash160);
    chronikClient.setUtxos(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path1899.hash160,
        apiError
            ? new Error('Error fetching utxos')
            : {
                  outputScript: `76a914${wallet.Path1899.hash160}88ac`,
                  utxos: wallet.state.nonSlpUtxos.concat(wallet.state.slpUtxos),
              },
    );

    // We set legacy paths to contain no utxos
    chronikClient.setScript(CASHTAB_ADDRESS_TYPE, wallet.Path145.hash160);
    chronikClient.setUtxos(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path145.hash160,
        apiError
            ? new Error('Error fetching utxos')
            : {
                  outputScript: `76a914${wallet.Path145.hash160}88ac`,
                  utxos: [],
              },
    );
    chronikClient.setScript(CASHTAB_ADDRESS_TYPE, wallet.Path245.hash160);
    chronikClient.setUtxos(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path245.hash160,
        apiError
            ? new Error('Error fetching utxos')
            : {
                  outputScript: `76a914${wallet.Path245.hash160}88ac`,
                  utxos: [],
              },
    );

    // TX history mocks
    chronikClient.setTxHistory(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path1899.hash160,
        apiError
            ? new Error('Error fetching history')
            : wallet.state.parsedTxHistory,
    );
    // We set legacy paths to contain no utxos
    chronikClient.setTxHistory(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path145.hash160,
        apiError ? new Error('Error fetching history') : [],
    );
    chronikClient.setTxHistory(
        CASHTAB_ADDRESS_TYPE,
        wallet.Path245.hash160,
        apiError ? new Error('Error fetching history') : [],
    );

    // Mock chronik.tx(tokenId) calls for tokens in tx history
    for (const tx of wallet.state.parsedTxHistory) {
        const mockedTokenResponse = {
            genesisInfo: tx.parsed.genesisInfo,
        };
        if (tx.parsed.isEtokenTx) {
            chronikClient.setMock('token', {
                input:
                    'tokenEntries' in tx.parsed
                        ? tx.parsed.tokenEntries[0].tokenId
                        : tx.parsed.slpMeta.tokenId,
                output: mockedTokenResponse,
            });
        }
    }
};

/**
 *
 * @param {object} chronikClient a mockedChronikClient object
 * @param {object} wallet a valid cashtab wallet object
 * @param {boolean} apiError true if we want to set api errors in mocked chronik
 * @returns modifies chronikClient in place to have expected API calls for wallet loading available
 */
export const prepareMockedChronikCallsForWallet = (
    chronikClient,
    wallet,
    apiError = false,
) => {
    // mock chronik endpoint returns
    const CASHTAB_TESTS_TIPHEIGHT = 800000;
    chronikClient.setMock('blockchainInfo', {
        output: apiError
            ? new Error('Error fetching blockchainInfo')
            : { tipHeight: CASHTAB_TESTS_TIPHEIGHT },
    });

    // Cashtab only supports p2pkh addresses
    const CASHTAB_ADDRESS_TYPE = 'p2pkh';

    // If you are mocking a legacy wallet to test a migration, return prepareMockedChronikCallsForLegacyWallet
    if (!('paths' in wallet)) {
        return prepareMockedChronikCallsForLegacyWallet(
            chronikClient,
            wallet,
            apiError,
        );
    }

    // Iterate over paths to create chronik mocks
    for (const path of wallet.paths) {
        // Mock scriptutxos to match context
        chronikClient.setScript(CASHTAB_ADDRESS_TYPE, path.hash);
        chronikClient.setUtxos(
            CASHTAB_ADDRESS_TYPE,
            path.hash,
            apiError
                ? new Error('Error fetching utxos')
                : {
                      outputScript: `76a914${path.hash}88ac`,
                      utxos:
                          path.path === 1899
                              ? wallet.state.nonSlpUtxos.concat(
                                    wallet.state.slpUtxos,
                                )
                              : [],
                  },
        );

        // Mock tx history
        chronikClient.setTxHistory(
            CASHTAB_ADDRESS_TYPE,
            path.hash,
            apiError
                ? new Error('Error fetching history')
                : path.path === 1899
                ? wallet.state.parsedTxHistory
                : // No tx history at legacy paths
                  [],
        );
    }

    // Mock chronik.tx(tokenId) calls for tokens in tx history
    for (const tx of wallet.state.parsedTxHistory) {
        const mockedTokenResponse = {
            genesisInfo: tx.parsed.genesisInfo,
        };
        if (tx.parsed.isEtokenTx) {
            chronikClient.setMock('token', {
                input:
                    'tokenEntries' in tx.parsed
                        ? tx.parsed.tokenEntries[0].tokenId
                        : tx.parsed.slpMeta.tokenId,
                output: mockedTokenResponse,
            });
        }
    }
};

/**
 * Remove all keys individually as localforage.clear() occasionally returns before all storage is clear
 * @param {object} localforage the localforage instance used in your test
 */
export const clearLocalForage = async localforage => {
    const SUPPORTED_CASHTAB_STORAGE_KEYS = [
        'cashtabCache',
        'contactList',
        'savedWallets',
        'settings',
        'wallet',
        'wallets',
    ];
    for (const key of SUPPORTED_CASHTAB_STORAGE_KEYS) {
        await localforage.removeItem(key);
    }
};
